>module ColorNesting where
>import Util
>import Specification
>import UsageElim
>import Maybe
>import PrettyP
>import List(groupBy)

This optimization looks for pairs of similar nodes which might be
possible to combine:

    A
   /|\
V B C B F
  |   |
  D   E

to:
    A
    |
  V B F
   /|\
  D C E
This can be only done, if C node does not use the features that B changes.

The task can be divided to the following parts:
  1) Find two equivalent nodes.
  2) Check whether C uses an features changed by B
  3) Transform

We do the change using convinient divide & conquer strategy to get speed
to the algorithm. Without divide&conquer, the algorithm uses huge amount
of time going through node lists...

>colornesting :: Rec CTree -> Rec CTree
>colornesting tree = fold checkreducablechilds tree

>checkreducablechilds :: CTree (Rec CTree) -> Rec CTree
>checkreducablechilds n |(nodetype n) == TagType = setchilds (shallow_colornesting_forest [] (child n)) (In n)
>checkreducablechilds x = In x

>colornesting_forest :: [Int] -> [Rec CTree]->[Rec CTree]
>colornesting_forest i lst = shallow_colornesting_forest i (map colornesting lst)

>shallow_colornesting_forest, shallow_colornesting_forest' :: [Int] -> [Rec CTree]->[Rec CTree]
>shallow_colornesting_forest i lst = concatMap (shallow_colornesting_forest' i) (groupBy similar lst)
>    where similar x y = ((nodetype x) == CharType) == ((nodetype y) == CharType)
>
>shallow_colornesting_forest' i lst = 
>            let
>            tags = map (\x->if nodetype x==TagType then Just (tag x) else Nothing) lst
>	     canwenest = findEqNodes i tags
>	     changed = colornestingchange i lst canwenest
>	  in changed

>colornestingchange :: [Int] -> [Rec CTree] -> (Int, Int) -> [Rec CTree]
>colornestingchange i tree (-1,-1) = tree
>colornestingchange _ tree (i1,i2) |

            debug ("A:", show anduses) $

>    allowedtodo = 
>		colornesting_forest [] (start ++ [newnode] ++ end)
>     where  allowedtodo = (not (b_usage anduses ||em_usage anduses ||i_usage anduses ||s_usage anduses ||tt_usage anduses || u_usage anduses ||size_usage anduses ||color_usage anduses))
>            uses = map (fold calcusage) middle
>            mergeuses = foldr mergeusage nousage uses
>            taguses = usage (nodecontext (head bnodeList)) (tag (head bnodeList)) nousage  
>            anduses = andusage mergeuses taguses
>            
>             
>            start = take i1 tree
>            end = drop (i2+1) tree
>            middle = take (i2-i1-1) (drop (i1+1) tree)
>            bnodeList = take 1 (drop i1 tree)
>	     bnodeList2 = take 1 (drop i2 tree)
>            newnode = setchilds ((child (head bnodeList)) ++ middle ++ (child (head bnodeList2))) (head bnodeList)



>colornestingchange i tree (i1,i2) = shallow_colornesting_forest (i1:i) tree


>findEqNodes :: [Int] -> [Maybe Tag] -> (Int,Int)
>findEqNodes i t = let
>          t3 = map (\x -> if (fst x) `elem` i then Nothing else snd x) indexes
>          count = countTags t3 []
>          possibletags = removeOnes count
>          firstpair :: [(Tag,Int)] -> (Maybe Tag, Maybe Int)
>	   firstpair ((x,y):_) = (Just x,Just y)
>          firstpair [] = (Nothing,Nothing)
>          indexes = zip [0..] t
>          indexlist = [(fst y) |y <- indexes, notElem (fst y) i, 
>                      isJust (snd y), (snd y) == (fst (firstpair possibletags))]
>
>          index_2 = take 2 indexlist
>	   (a,b) = case index_2 of
>		[a,b] -> (a,b)
>		_ -> (-1,-1)
>        in 

         debug ("Possible: ", (possibletags,indexlist, index_2),i) $ 

>         (a,b)         

>removeOnes :: [(Tag, Int)] -> [(Tag,Int)]
>removeOnes ((_,1) : xs) = removeOnes xs
>removeOnes ((t,v) : xs) = (t,v) : removeOnes xs
>removeOnes [] = []

>countTags :: [Maybe Tag] -> [(Tag, Int)] -> [(Tag, Int)]
>countTags (Nothing:xs) a = countTags xs a
>countTags ((Just x):xs) a = countNodes x (countTags xs a)
>countTags [] a = []

>countNodes :: Tag -> [(Tag,Int)] -> [(Tag, Int)]
>countNodes t ((a,i):xs) |t == a = ((t,i+1):xs)
>countNodes t ((s,i):xs) = ((s,i):countNodes t xs)
>countNodes t [] = [(t,1)]











