>module CommonParser (CommonParser, module Parser) where
>import Parser
>import Monad (MonadPlus(..))
>import Arrow
>import PrettyP
>import Util (debug)
>
>data CommonParser c a b = MakeParser 
>     { parseFunc :: [c] -> a -> ParserResult c b,
>	parseAnnotation :: Doc
>     }


>instance Arrow (CommonParser c) where
>	  arr f = MakeParser ff $ pp "{}"
>		where ff inp ed = ParseOk (f ed) inp emptyParsed
>         x >>> y = MakeParser ff $ fsep[annotation x,pp ">>>",annotation y]
>		where ff inp ed = case parseFunc x inp ed of
>			 ParseOk v r p -> case parseFunc y r v of
>				 ParseOk v' r' p' -> ParseOk v' r' (p . p')
>				 ParseFail m p' -> ParseFail m (p . p')
>			 ParseFail m p -> ParseFail (m $+$ pp "at"
>						      <+> annotation x) p
>         first x = MakeParser ff $ pp "first" <+> annotation x
>		where ff inp ~(a,b) = case parseFunc x inp a of
>			 ParseOk v r p -> ParseOk (v,b) r p
>			 ParseFail m p -> ParseFail (m $+$ pp "at" <+> annotation x) p

>instance ArrowZero (CommonParser c) where
>	  zeroArrow = MakeParser ff $ pp "zero"
>		where ff _ _ = ParseFail (pp "Parse failure: failure parser used")
>					 emptyParsed

>instance (PpShow c,PpShow [c]) => ArrowPlus (CommonParser c) where
>	  x +++ y = x ==|= y

>instance PpShow c => ArrowChoice (CommonParser c) where
>	  left x = MakeParser ff $ pp "left" <+> annotation x
>	    where ff inp z = case z of
>		     Left ed	  -> fmap Left $ parseFunc x inp ed
>		     Right ed'  -> ParseOk (Right ed') inp emptyParsed


>instance ArrowApply (CommonParser c) where
>	  app = MakeParser ff $ pp "app"
>	      where ff inp ~(p,i) = parseFunc p inp i


>mapResult :: ([b] -> ParserResult b a) -> [b] -> [a]
>mapResult f s = case f s of
>	   ParseOk v r _ -> (v:mapResult f r)
>	   ParseFail _ _ -> []

>verbose::Bool

>verbose = True

verbose = False 




>instance Show (CommonParser c a b) where
>	  showsPrec _ x = showString $ render $ pp x

>instance PpShow (CommonParser c a b) where
>	  pp x = parseAnnotation x

>instance AnnotatedParser CommonParser where
>	  annotateP z n = commonAnnotate z n
>	  primitiveAnnotateP z n = z { parseAnnotation = n }
>	  annotationP x = parseAnnotation x
>	  primitiveAnnotate z n = MakeParser ff n
>		where ff inp a = case parseFunc z inp a of
>			 z@(ParseOk _ _ _) -> z -- debug n z -- z -- debug n z
>			 ParseFail m p -> -- debug ("fail:",n) $ 
>				 ParseFail 
>				   (if verbose then enclose n $$ pp (take 10 inp) 
>				       $$ m  else
>				       fsep[pp "Expected:", enclose n, 
>					       pp "got:", pp (take 10 inp)]) p

primitiveAnnotateP z n

	     

>instance ExtraParser CommonParser where
>	  getExtraData = MakeParser f $ pp "getExtraData"
>		       where f inp ed = ParseOk ed inp emptyParsed
>	  setExtraData _ = MakeParser f $ pp "setExtraData"
>		       where f inp _ = ParseOk () inp emptyParsed

>instance AEParser CommonParser

>instance Parser CommonParser where
>	  (=|=) = commonOrParser
>	  (==|=) = commonLeftOrParser
>	  (==>)  = commonChainParser
>	  consumeP = commonConsumeParser 
>	  succeedP = commonSucceedParser
>	  mapP = commonMapParser
>	  catchP = commonCatchParser
>         inputOf = MakeParser (\inp _ -> ParseOk inp inp emptyParsed) $ 
>				     pp "inputOf"
>	  parseP (MakeParser x _) ed inp = return $ x inp ed

>instance (ErrorInfo e,PpShow c) => Monad (CommonParser c e) where
>	  (>>=) = commonBindParser
>	  return = commonUnitParser
>	  fail x = commonFailParser $ pp x

>commonUnitParser :: a -> CommonParser c e a
>commonUnitParser a = MakeParser (\inp _ -> ParseOk a inp emptyParsed) $ pp "<>"



>instance (ErrorInfo e,PpShow c, PpShow [c]) => MonadPlus (CommonParser c e) where
>	  a `mplus` b = a ==|= b
>	  mzero = fail "Parse error"

>instance (ErrorInfo e,PpShow c, PpShow [c],Ord c) => MonadicParser CommonParser c e

>commonSucceedParser :: (PpShow [c],PpShow c, PpShow a) => 
>			ParserResult c a -> CommonParser c e a
>commonSucceedParser pres = MakeParser ff (pp "<succeed:" <> pp pres <> pp ">")
>		     where ff _ _ = pres

>commonMapParser :: (ParserResult c a -> ParserResult c b) ->
>		    CommonParser c e a -> CommonParser c e b
>commonMapParser f (MakeParser a r) = MakeParser ff r
>		 where ff inp ed = f (a inp ed)

>{-# INLINE commonBindParser #-}
>commonBindParser :: CommonParser c e a
>		     -> (a -> CommonParser c e b)
>		     -> CommonParser c e b
>commonBindParser ~(MakeParser a r) f = MakeParser ff r
>      where ff inp ed = case (a inp) ed of
>		ParseOk x rest p -> case (parseFunc (f x)) rest ed of
>			ParseOk x' rest' p' -> ParseOk x' rest' (p . p')
>			ParseFail m p' -> ParseFail m (p . p')
>		ParseFail msg p -> ParseFail msg p

>commonAnnotate :: CommonParser c e a -> Doc -> CommonParser c e a
>commonAnnotate (MakeParser a r) ann = MakeParser ff $ ann
>		where ff inp ed = case a inp ed of
>			 z@(ParseOk _ _ _) -> z
>			 ParseFail msg p -> ParseFail (if verbose then msg $+$ msg2 p else msg) p
>		      msg2 p' = nest 5 (fsep[pp "in", ann, pp "::=",pp r])



>commonFailParser :: (ErrorInfo e, PpShow c) => Doc -> CommonParser c e a
>commonFailParser msg = MakeParser f $ an
>			where an = pp "<error:" <> msg <> pp ">"
>			      f inp ed = ParseFail (messageOf ed inp msg) emptyParsed

>commonCatchParser :: CommonParser c e a 
>		      -> (Doc -> CommonParser c e a) 
>		      -> CommonParser c e a
>a `commonCatchParser` f = MakeParser qf $ (annotationP a)
>	     where qf inp ed = case (parseFunc a) inp ed of
>			 x@(ParseOk _ _ _)	-> x
>			 ParseFail msg p -> case (parseFunc (f msg)) inp ed of
>				   z@(ParseOk x' r' p') -> ParseOk x' r' p'
>				   ParseFail m p' -> ParseFail m (maxlist p p')

>maxlist :: ([a] -> [a]) -> ([a] -> [a]) -> ([a] -> [a])
>maxlist x y = if (length (x [])) > (length (y [])) then x else y


>commonConsumeParser :: (ErrorInfo e, PpShow c) => CommonParser c e c
>commonConsumeParser = MakeParser f $ pp "<.>"
>	 where f (cc:r) _ = ParseOk cc r (\rr -> cc : rr)
>	       f inp ed	= ParseFail (messageOf ed inp (pp "Unexpected EOF")) emptyParsed

>commonOrParser :: (ErrorInfo e, PpShow c, PpShow a, PpShow [c]) =>
>		CommonParser c e a -> CommonParser c e a -> CommonParser c e a
>p `commonOrParser` q = MakeParser f $ an 
>	   where qf = parseFunc q
>		 pf = parseFunc p 
>		 f inp ed = case (pf inp ed,qf inp ed) of
>		      (z@(ParseOk x' inp' px'), (ParseFail _ py')) -> ParseOk x' inp' px'
>		      ((ParseFail _ py'), z@(ParseOk x' inp' px')) -> ParseOk x' inp' px'
>		      ((ParseFail m1 px),(ParseFail m2 px')) -> 
>				  if length (px []) > length (px' []) then
>				         ParseFail m1 px 
>				    else ParseFail m2 px'
>		      (z@(ParseOk x inp' px), z'@(ParseOk y _ px')) -> 
>				if length (px []) >= length (px' []) then z else z'

ParseFail
				(messageOf ed inp'
			          (ambiguousError inp p q x y)
						 $+$ errorDtls px px') (maxlist px px')

>	         an = fsep[annotation p,pp "|",annotation q]
>		 errorDtls px px' = if verbose then pp "at" <+> pp (maxlist px px') else pp ""

The following is a one-sided version, which never produces ambiguousness
problems:

>commonLeftOrParser :: (PpShow [c],PpShow c) => CommonParser c e a
>			 -> CommonParser c e a
>			 -> CommonParser c e a
>p `commonLeftOrParser` q = MakeParser f $ an
>	where f inp ed = case pf inp ed of
>		z@(ParseOk _ _ p1) -> z
>		ParseFail m1 px -> case qf inp ed of
>			   (ParseOk x' inp' px') -> ParseOk x' inp' px'
>			   (ParseFail m2 px') -> if (length (px []) >= length (px' [])) then
>				      ParseFail m1 px
>				   else
>				      ParseFail m2 px'

ParseFail (

			     annotation q <+> pp (maxlist px px') <+> pp ':' <+> 
			     if verbose then m2 $+$ annotation p <+> pp ':' <+> m1
				else m2) (maxlist px px')

>	      qf = -- debug ((render (annotation p)) ++ "=q|=" ++ render (annotation q)) $ 
>		   parseFunc q
>	      pf = --debug ((render (annotation p)) ++ "==|=" ++ render (annotation q)) $ 
>		   parseFunc p
>	      an = fsep[annotation p,pp "|",annotation q]
>	      errorDtls = if verbose then pp "at" <+> an else pp ""

The equivalent of (.) for parsers. p ==> q feeds the output of the
first parser as an input to the second parser, and returns the result.
Note the result type of the first parser must be a list of items.
Also, this operator fails, if all input (of either parser) was not
consumed - then a parse failure is signalled.


>commonChainParser :: (PpShow a, PpShow c, PpShow [a],PpShow b,ErrorInfo e) =>
>		      CommonParser c e [a]
>		      -> CommonParser a e b
>		      -> CommonParser c e b
>a `commonChainParser` b = MakeParser f $ ann
>	 where ann = (fsep[annotation a, pp "==>" , annotation b])
>	       errorDtls = if verbose then pp "at3" <+> ann else pp ""
>	       f inp ed = case parseFunc a inp ed of
>		 (ParseOk z [] p) -> case parseFunc b z ed of
>			 (ParseOk r1 [] _) -> (ParseOk r1 [] p)
>			 (ParseOk r1 (c:d) p') -> (ParseFail
>				 (fsep[ 
>				 pp "Unexpected input '"
>				 <> hsep (map pp (take 10 (c:d)))
>				 <> pp "'."
>				 ,pp "After reading:",pp r1]) p)
>			 (ParseFail m1 pxx) -> ParseFail (
>					     m1 
>					     $$ pp "after reading:" $$ nest 5 (enclose (fsep (map pp (getParsed pxx))))
>					     $$ pp "got:" <+> pp z
>					     $$ errorDtls) p
>						 
>	         (ParseOk _ i@(c:_) p) -> (ParseFail $
>			 (messageOf ed i $
>			 pp "Unexpected input '" <> pp c
>			 <> pp "'.") $$ errorDtls) $ p
>		 (ParseFail m1 p) -> (ParseFail (m1 $$ errorDtls) p)

>ambiguousError :: (PpShow a, PpShow c) =>
>		[c] -> CommonParser c e a -> CommonParser c e a -> a -> a 
>		       -> Doc
>ambiguousError inp p1 p2 _ _ = pp "Ambiguous parse on input" <+> pp "'"
>			  <> cat (map pp (take 5 inp)) <> pp "'" 
>			  $+$ pp "Alternatives:" $+$ pp "1)" <+>
>			  annotation p1 $+$ pp "2)" <+>
>			  annotation p2


ambiguousError :: (Show a, CharShow c) => 
		[c] -> CommonParser c a -> CommonParser c a -> a -> a
		    -> ExtraParseData -> String
ambiguousError inp p1 p2 v1 v2 = messageOf ed inp $
		       "Ambiguous parse on input '"
		    ++ show (take 5 inp)
		    ++ "'\nAlternatives:\n1) "
		    ++ annotation p1 ++ " [" ++ (show v1) ++ "]\n2) "
		    ++ annotation p2 ++ " [" ++ (show v2) ++ "]"






