>module DocParser (pdocument,isSpaceChar, tokenToTree,Token(..)) where
>
>import Language
>import Parser
>import Monad(guard)
>import Arrow
>import Char (ord)
>import Util
>import CommonParser

>starttag :: MonadicParser p Char e => p Char e Token
>starttag = do (_,tag,_) <- do3 (charParser '<') (spanParser (/= '>'))
>				(charParser '>')
>	       guard (tag `elem` allowedTags)
>	       return (TStartTag tag)

>endtag :: MonadicParser p Char e => p Char e Token
>endtag = do (_,_,ctag,_) <- do4 (charParser '<') (charParser '/')
>					     (spanParser (/= '>'))
>					     (charParser '>')
>	     guard (ctag `elem` allowedTags)
>	     return $ TEndTag ctag

>data Token = TStartTag String
>	    | TEndTag String
>	    | TText String
>	    | TSpace String

>instance PpShow Token where
>	  pp (TStartTag s) = pp '<' <> pp s <> pp '>'
>	  pp (TEndTag s) = pp '<' <> pp '/' <> pp s <> pp '>'
>	  pp (TText s) = pp s
>	  pp (TSpace s) = pp s

>tokenize :: MonadicParser p Char e => p Char e [Token]
>tokenize = repPlus (starttag ==|= endtag ==|= textchar ==|= pspace)
>	    ==|= do inp <- inputOf
>		    guard (null inp)
>		    return []



>textchar :: MonadicParser p Char e => p Char e Token
>textchar = (spanParserPlus isTextChar >>> arr TText)
>	      `primitiveAnnotate` pp "textchar*"
>	  where isTextChar '<' = False
>		isTextChar '>' = False
>		isTextChar x = isPrintChar x && not(isSpaceChar x)

>isPrintChar :: Char -> Bool
>isPrintChar c | ord(c) >= 0x21 && ord(c) <= 0x7e = True
>	       | otherwise = False

>isSpaceChar :: Char -> Bool
>isSpaceChar c = c `elem` " \r\n\t"

>pspace :: MonadicParser p Char e => p Char e Token
>pspace = spanParserPlus isSpaceChar >>> arr TSpace
>	`primitiveAnnotate` pp "<whitespace>"



pseq :: (MonadicParser p c e, PpShow a,PpShow [a])
     => p c e a -> p c e (Document a)
pseq pdoc = do dlst <- rep pdoc
		return (DSeq dlst)
  `primitiveAnnotate` pp "document document"

prec :: (MonadicParser p c e) =>
     p c e (f (Rec f)) -> p c e (Rec f)
prec p = p >>> arr In
  `primitiveAnnotate` (annotation p)


>pdocument :: MonadicParser p Char e => p Char e [Token]
>pdocument = do toks <- tokenize
>		return toks

		results <- tokenToTree [] toks []
		return $ In $ DSeq results

>{-# SPECIALIZE pdocument :: CommonParser Char LineInfo [Token] #-}


pdocument :: MonadicParser p Char e => p Char e RDocument 
pdocument = (prec $ pseq $ prec (ptag pdocument
			 ==|= textchar
			 ==|= pspace))
	`primitiveAnnotate` pp "document"

Note the subtle use of "reverse". We have the following conventions:

  parent
    |                            /----toks------\
 a b c d e <tag> (a b c d e f) | </tag> a b c d e
 \-------/ \---/ \-----------/ ^ \----/ \-------/
   prev      t     forest      |   s     rest_of_file
                               |
                            current pos

Forest and prev are represented in reverse.

Arguments are:      ((t,prev):parent)     toks    forest

>tokenToTree :: Monad m => [(String,[RDocument])] 
>			   -> [Token] -> [RDocument] -> m [RDocument]
>tokenToTree ((t,_):re) [] _ = fail $ "Extra open tag:" ++ t
>tokenToTree ((t,prev):re) ((TEndTag s):rest_of_file) forest 
>	     | t == s = do let res = In (DTag t (In (DSeq (reverse forest))))
>			   tokenToTree re rest_of_file (res:prev)
>	     | otherwise = fail $ "mismatched tags:"
>						   ++ t ++ " vs. " ++ s
>tokenToTree x ((TStartTag st):rest_of_file) forest = 
>	       tokenToTree ((st,forest):x) rest_of_file []
>tokenToTree [] [] forest = return (reverse forest)
>tokenToTree [] ((TEndTag s):_) forest = fail $ "Extra close tag:" ++ s
>tokenToTree lst ((TText s):re) forest = tokenToTree lst re
>					   (In (DText s) : forest)
>tokenToTree lst ((TSpace s):re) forest = tokenToTree lst re
>					   (In (DSpace s):forest)
