>module Timer (start_processing, send_result, TimerHandle) where
>import Concurrent
>import Exception
>import IO
>data TimerHandle a = NTH (MVar a)

>send_result :: TimerHandle a -> a -> IO ()
>send_result (NTH h) v = do _ <- swapMVar h $! v
>			    return ()

>start_processing :: Integer -> a -> (TimerHandle a -> IO a) -> IO a
>start_processing timeoutval init f = do 
>		       resultVar <- newMVar init
>		       ready <- newQSem 0
>		       tid <- forkIO (processing_thread f ready (NTH resultVar))
>		       timerThread <- forkIO (timer_thread ready timeoutval tid)
>		       waitQSem ready
>		       result <- readMVar resultVar
>		       return result

>processing_thread :: (TimerHandle a -> IO a) -> QSem -> TimerHandle a -> IO ()
>processing_thread f ready handle = (catchJust asyncExceptions (do
>				       res <- f handle
>				       send_result handle res
>				       signalQSem ready)
>				       (\_ -> return ()))
>				    `Exception.catch` (\m -> do hPutStrLn stderr (show m)
>								signalQSem ready
>								return ())

>timer_thread :: QSem -> Integer -> ThreadId -> IO ()
>timer_thread sem i tid = do 
>			 threadDelay (fromIntegral (i*900000)) -- 10% for output
>			 killThread tid
>			 signalQSem sem


