>module Util where
>import PrettyP
>import IOExts (trace)

>instance (Functor f, PpShow (f Doc)) => PpShow (Rec f) where
>	  pp x = fold pp x

>instance (Functor f, Show (f ShowS)) => Show (Rec f) where
>	  showsPrec _ x = fold shows x

>instance Show ShowS where
>	  showsPrec _ x = x

>data Rec f = In ! (f (Rec f))

The "Rec" and folds are directly from [Meijer&Hutton]: "Bananas in
Space: Extending fold and unfold to exponential types".

>fold :: Functor f => (f a -> a) -> (Rec f -> a)
>fold phi (In x) = phi (fmap (fold phi) x)

>unfold :: Functor f => (a -> f a) -> a -> Rec f
>unfold psi x = In (fmap (unfold psi) (psi x))

>transform :: Functor f => (b -> f b) -> (f a -> a) -> b -> a
>transform construct destruct = fold destruct . unfold construct

>maphierarchy :: (Functor f, Functor g) => (f a -> a) -> (b -> g b) -> (a->b) -> (Rec f -> Rec g)
>maphierarchy destruct construct trans = unfold construct . trans . fold destruct

>debugM_ :: (PpShow b, Monad m) => b -> m a -> m a
>debugM_ s x = debug s x >>= \val -> debug ("result",s) $ return val

>debug :: (PpShow s) => s -> a -> a
>debug s = trace $ render $ pp "debug:" <+> pp s

>debugM :: (PpShow b,Monad m, PpShow a) => b -> m a -> m a
>debugM s x = debug s $ x >>= \val -> debug ("result:",s,val) $ return val
